package jeuDeLavie;

import java.io.*;
import java.util.*;

public class JeuDeLaVie {
    private Environnement environnement;
    private AfficheGrille visualisation;
    private static int nbLigne = 20, nbColonne = 20, nbCycle = 100;
    private static boolean infini = true;

    public JeuDeLaVie(int lignes, int colonnes) {
        environnement =  new Environnement(lignes, colonnes);
    }
    
    public void setAfficheGrille(AfficheGrille visualisation) {
        this.visualisation = visualisation;
    	if (environnement != null) {
    	    environnement.setAfficheGrille(visualisation);
    	}	
    }	

    private void executeOneCycle()
    {
        Environnement generationSuivante  = new Environnement(environnement.getNbLignes(), 
                environnement.getNbColonnes(), visualisation);
        // On implmente les rgles et on dtermine la gnration suivante
        for(int i = 0; i < environnement.getNbLignes(); i++)
            for(int j = 0; j < environnement.getNbColonnes(); j++)
            {
                Position p = new Position(i, j);
                int nbreVoisins = environnement.nombreDeVoisins(p);
                if (nbreVoisins < 2 || nbreVoisins > 3)
                    generationSuivante.setCelluleEn(p, false);
                else if (nbreVoisins == 3)
                    generationSuivante.setCelluleEn(p, true);
                else generationSuivante.setCelluleEn(p, environnement.getCelluleEn(p).getEtat());
            }
        environnement = generationSuivante;
    }
    
    public void execute(int nbSteps, boolean infini)
    {	  
        for(int i = 0; i < nbSteps || infini; i++)
        {
            executeOneCycle();
            visualisation.affiche(environnement, "generation "+i);
        }
    }
    
    
    public void ficInit(String nomDuFichier)
    {   
    	try {
    	    	FileReader fr = new FileReader(nomDuFichier);
    	    	BufferedReader br = new BufferedReader(fr);
    	    	String s = br.readLine();
    	    	while (br.ready())
    	    	{
    	    	    s = br.readLine();
    	    	    StringTokenizer st = new StringTokenizer(s," ", false);
    	    	    environnement.setCelluleEn(
    	                        new Position(Integer.parseInt(st.nextToken()), 
    	                                Integer.parseInt(st.nextToken())), true); 
    	    	}
    	    	br.close();
    		}   catch(Exception e)
    			{
    		    e.printStackTrace();
    			}
    }
    
    private static void lectureTailleGrille(String nomDuFichier)
    {   
    	try {
    	    	FileReader fr = new FileReader(nomDuFichier);
    	    	BufferedReader br = new BufferedReader(fr);
    	    	String s = br.readLine();
    	    	StringTokenizer st = new StringTokenizer(s," ", false);
    	    	nbLigne = Integer.parseInt(st.nextToken());
    	    	nbColonne = Integer.parseInt(st.nextToken());
    	    	if (st.hasMoreTokens()) 
    	    		{
    	    	    	nbCycle = Integer.parseInt(st.nextToken());
    	    	        infini = false;
    	    	    }
    	    	br.close();
    		}   catch(Exception e)
    			{
    		    e.printStackTrace();
    			}
    }
    
    public void randomInit()
    {
        for(int i = 0; i < environnement.getNbLignes(); i++)
            for(int j = 0; j < environnement.getNbColonnes(); j++)
                    environnement.setCelluleEn(new Position(i, j), Math.random() < 0.5);        
        environnement.setCelluleEn(new Position(5, 7), true);
        environnement.setCelluleEn(new Position(6, 7), true);
        environnement.setCelluleEn(new Position(7, 7), true);
        
    }
    
    static public void main(String[] s)
    {
        String usage = "Usage : java jeuDeLavie.JeuDeLaVie -f nom_de_fichier\n" +
        "        java jeuDeLavie.JeuDeLaVie -lg nombre_de_ligne -col nombre_de_colonne" +
        " [-nbcycle nombr_de_cycle]";
        String nomFic = "";   
        int i = 0, l = s.length;
        
        if (l != 2 && l != 4 && l != 6 ) {            
            System.err.println(usage);
            System.exit(0);
        }
        try {
            while(i < l)
            {
                if (l == 2 && s[i].equals("-f"))
                {
                    nomFic = s[i+1];
                    lectureTailleGrille(nomFic);
                }
                	else 
                	{
                	    if (s[i].equals("-lg")) 
                	        nbLigne = Integer.parseInt(s[i+1]);
                	    	else if (s[i].equals("-col")) 
                	    	    	nbColonne = Integer.parseInt(s[i+1]);
                	    			else if (l == 6 && s[i].equals("-nbcycle"))
                	    					{
                	    			    		nbCycle = Integer.parseInt(s[i + 1]);
                	    			    		infini = false;
                	    					}
                	    					else 
                	    						{           
                	    					    	System.err.println(usage);
                	    					    	System.exit(0);   
                	    						}
                	}
                i+=2;
            }
        } catch(Exception e) 
        	{
            	System.err.println(e); 
            	System.err.println(usage); 
            	System.exit(0);
            }
        //AfficheGrille visu = new AfficheurGrilleGra(nbLigne,nbColonne); 
        AfficheGrille visu = new AfficheurGrilleTexte(true); 
        JeuDeLaVie jdv = new JeuDeLaVie(nbLigne,nbColonne);
        jdv.setAfficheGrille(visu);
        if (nomFic.equals(""))
            jdv.randomInit();
        	else jdv.ficInit(nomFic);
         jdv.visualisation.affiche(jdv.environnement, "generation initiale");
         jdv.execute(nbCycle, infini);
    }
}
